@@ -66,7 +66,7 @@ static int cache_index(char *filename) {
 		c1 = *c;
 		if ((c1 >= 'A') && (c1 <= 'Z')) {
 			c1 -= ('A' - 1);
-		} else if ((c1 >= 'a') && (c1 <= 'a')) {
+		} else if ((c1 >= 'a') && (c1 <= 'z')) {
 			c1 -= ('a' - 1);
 		} else {
 			c1 = 0;
@@ -79,7 +79,7 @@ static int cache_index(char *filename) {
 		c2 = *(c + 1);
 		if ((c2 >= 'A') && (c2 <= 'Z')) {
 			c2 -= ('A' - 1);
-		} else if ((c2 >= 'a') && (c2 <= 'a')) {
+		} else if ((c2 >= 'a') && (c2 <= 'z')) {
 			c2 -= ('a' - 1);
 		} else {
 			c2 = 0;
