@@ -306,10 +306,10 @@ int main(int argc, char *argv[]) {
 	}
 	follow_symlinks = (strcmp(var, "true") == 0);
 
-	if ((var = getenv(cgiwrap_cgitime)) != NULL) {
-		time_for_cgi = str2int(var);
-	} else {
+	if ((var = getenv(cgiwrap_cgitime)) == NULL) {
 		time_for_cgi = 5;
+	} else if ((time_for_cgi = str2int(var)) == -1) {
+		print_code(500);
 	}
 
 	if ((var = getenv(cgiwrap_userdir)) != NULL) {
