@@ -16,12 +16,12 @@
 #include <stdbool.h>
 #include <unistd.h>
 #include <string.h>
-#include <limits.h>
 #include <fcntl.h>
 #include <time.h>
 #include <signal.h>
 #include <dirent.h>
 #include <errno.h>
+#include <limits.h>
 #include <pthread.h>
 #include <syslog.h>
 #include <grp.h>
@@ -72,12 +72,6 @@
 
 #define REQUEST_BUFFER_CHUNK     4 * KILOBYTE
 
-#define fe_MAX_REQUESTSIZE      -2
-#define fe_TIMEOUT              -3
-#define fe_CLIENT_DISCONNECTED  -4
-#define fe_READ_ERROR           -5
-#define fe_FORCE_QUIT           -6
-
 #define rs_NONE                  0
 #define rs_QUIT_SERVER           1
 #define rs_UNBAN_CLIENTS         2
@@ -89,6 +83,8 @@
 #define NO_REQUEST_LIMIT_TIME  300
 #define NO_REQUEST_LIMIT_SIZE   16 * MEGABYTE
 
+#define LOG_PERM (S_IRUSR|S_IWUSR|S_IRGRP)
+
 typedef struct {
 	char *config_dir;
 	bool daemon;
@@ -139,15 +135,15 @@ char *version_string = "Hiawatha v"VERSION
 void touch_logfiles(t_config *config, char *dir) {
 	t_host *host;
 
-	touch_logfile(dir, config->system_logfile, 0640, config->server_uid, config->server_gid);
+	touch_logfile(dir, config->system_logfile, LOG_PERM, config->server_uid, config->server_gid);
 	if (config->garbage_logfile != NULL) {
-		touch_logfile(dir, config->garbage_logfile, 0640, config->server_uid, config->server_gid);
+		touch_logfile(dir, config->garbage_logfile, LOG_PERM, config->server_uid, config->server_gid);
 	}
 	if (config->exploit_logfile != NULL) {
-		touch_logfile(dir, config->exploit_logfile, 0640, config->server_uid, config->server_gid);
+		touch_logfile(dir, config->exploit_logfile, LOG_PERM, config->server_uid, config->server_gid);
 	}
 #ifdef DEBUG
-	touch_logfile(dir, LOG_DIR"/debug.log", 0640, config->server_uid, config->server_gid);
+	touch_logfile(dir, LOG_DIR"/debug.log", LOG_PERM, config->server_uid, config->server_gid);
 #endif
 
 	host = config->first_host;
@@ -155,8 +151,8 @@ void touch_logfiles(t_config *config, char *dir) {
 		if (host->access_fileptr != NULL) {
 			fflush(host->access_fileptr);
 		}
-		touch_logfile(dir, host->access_logfile, 0640, config->server_uid, config->server_gid);
-		touch_logfile(dir, host->error_logfile, 0640, config->server_uid, config->server_gid);
+		touch_logfile(dir, host->access_logfile, LOG_PERM, config->server_uid, config->server_gid);
+		touch_logfile(dir, host->error_logfile, LOG_PERM, config->server_uid, config->server_gid);
 		host = host->next;
 	}
 }
@@ -396,7 +392,7 @@ int fetch_request(t_session *session) {
 							}
 						}
 						if (session->uploaded_file == NULL) {
-							log_error(session, "can't create tempfile for PUT request");
+							log_error(session, "can't create temporary file for PUT request");
 							result = 500;
 							break;
 						}
@@ -420,7 +416,7 @@ int fetch_request(t_session *session) {
 							content_length = str2int(strstart);
 							*strend = '\r';
 							if ((content_length < 0) || (INT_MAX - content_length - 2 <= header_length)) {
-								result = 400;
+								result = 500;
 								break;
 							}
 
@@ -435,13 +431,15 @@ int fetch_request(t_session *session) {
 								if ((new_reqbuf = (char*)realloc(session->request, session->buffer_size + 1)) != NULL) {
 									session->request = new_reqbuf;
 								} else {
-									result = fe_READ_ERROR;
+									session->error_cause = ec_SOCKET_READ_ERROR;
+									result = -1;
 									break;
 								}
 							} else {
 								session->content_length = content_length;
 								if (header_length + content_length > max_request_size) {
-									result = fe_MAX_REQUESTSIZE;
+									session->error_cause = ec_MAX_REQUESTSIZE;
+									result = -1;
 									break;
 								}
 
@@ -450,7 +448,8 @@ int fetch_request(t_session *session) {
 									if ((new_reqbuf = (char*)realloc(session->request, session->buffer_size + 1)) != NULL) {
 										session->request = new_reqbuf;
 									} else {
-										result = fe_READ_ERROR;
+										session->error_cause = ec_SOCKET_READ_ERROR;
+										result = -1;
 										break;
 									}
 								}
@@ -487,16 +486,19 @@ int fetch_request(t_session *session) {
 		switch (select(session->client_socket + 1, &read_fds, NULL, NULL, &select_timeout)) {
 			case -1:
 				if (errno != EINTR) {
-					result = fe_READ_ERROR;
+					session->error_cause = ec_SOCKET_READ_ERROR;
+					result = -1;
 					keep_reading = false;
 				}
 				break;
 			case 0:
 				if (session->force_quit) {
-					result = fe_FORCE_QUIT;
+					session->error_cause = ec_FORCE_QUIT;
+					result = -1;
 					keep_reading = false;
 				} else if (time(NULL) > deadline) {
-					result = fe_TIMEOUT;
+					session->error_cause = ec_TIMEOUT;
+					result = -1;
 					keep_reading = false;
 				}
 				break;
@@ -506,7 +508,8 @@ int fetch_request(t_session *session) {
 					if ((new_reqbuf = (char*)realloc(session->request, session->buffer_size + 1)) != NULL) {
 						session->request = new_reqbuf;
 					} else {
-						result = fe_READ_ERROR;
+						session->error_cause = ec_SOCKET_READ_ERROR;
+						result = -1;
 						keep_reading = false;
 						break;
 					}
@@ -526,12 +529,14 @@ int fetch_request(t_session *session) {
 				switch (bytes_read) {
 					case -1:
 						if (errno != EINTR) {
-							result = fe_READ_ERROR;
+							session->error_cause = ec_SOCKET_READ_ERROR;
+							result = -1;
 							keep_reading = false;
 						}
 						break;
 					case 0:
-						result = fe_CLIENT_DISCONNECTED;
+						session->error_cause = ec_CLIENT_DISCONNECTED;
+						result = -1;
 						keep_reading = false;
 						break;
 					default:
@@ -563,7 +568,8 @@ int fetch_request(t_session *session) {
 
 							if (session->bytes_in_buffer > max_request_size) {
 								keep_reading = false;
-								result = fe_MAX_REQUESTSIZE;
+								session->error_cause = ec_MAX_REQUESTSIZE;
+								result = -1;
 								break;
 							}
 						}
@@ -952,9 +958,11 @@ int serve_client(t_session *session) {
 	/* URL toolkit
 	 */
 #ifdef HAVE_SSL
-	init_toolkit_options(&options, session->host->website_root, session->config->url_toolkit, session->binding->use_ssl);
+	init_toolkit_options(&options, session->host->website_root, session->config->url_toolkit,
+	                     session->binding->use_ssl, session->headerfields);
 #else
-	init_toolkit_options(&options, session->host->website_root, session->config->url_toolkit, false);
+	init_toolkit_options(&options, session->host->website_root, session->config->url_toolkit,
+	                     false, session->headerfields);
 #endif
 
 	if ((session->request_method != PUT) && (session->request_method != DELETE)) {
@@ -1099,13 +1107,17 @@ int serve_client(t_session *session) {
 			}
 	}
 
-	length = strlen(session->file_on_disk);
-	if (*(session->file_on_disk + length - 1) == '/') {
-		if (session->uri_is_dir) {
+#ifdef HAVE_TOOLKIT
+	if ((session->toolkit_fastcgi == NULL) && session->uri_is_dir) {
+#else
+	if (session->uri_is_dir) {
+#endif
+		length = strlen(session->file_on_disk);
+		if (*(session->file_on_disk + length - 1) == '/') {
 			strcpy(session->file_on_disk + length, session->host->start_file);
+		} else {
+			return 301;
 		}
-	} else if (session->uri_is_dir) {
-		return 301;
 	}
 
 	if (get_target_extension(session) == -1) {
@@ -1201,8 +1213,8 @@ void handle_timeout(t_session *session) {
 /* Request has been handled, handle the return code.
  */
 void handle_request_result(t_session *session, int result) {
-	switch (result) {
-		case fe_MAX_REQUESTSIZE:
+	if (result == -1) switch (session->error_cause) {
+		case ec_MAX_REQUESTSIZE:
 			log_system(session, "Maximum request size reached");
 			session->return_code = 413;
 			send_code(session);
@@ -1216,27 +1228,30 @@ void handle_request_result(t_session *session, int result) {
 #endif
 			}
 			break;
-		case fe_TIMEOUT:
+		case ec_TIMEOUT:
 			if (session->kept_alive == 0) {
 				session->return_code = 408;
 				send_code(session);
 				handle_timeout(session);
 			}
 			break;
-		case fe_CLIENT_DISCONNECTED:
+		case ec_CLIENT_DISCONNECTED:
 			if (session->kept_alive == 0) {
 				log_system(session, "Client disconnected");
 			}
 			break;
-		case fe_READ_ERROR:
+		case ec_SOCKET_READ_ERROR:
 			if (errno != ECONNRESET) {
 				log_system(session, "Error while reading request");
 			}
 			break;
-		case fe_FORCE_QUIT:
+		case ec_SOCKET_WRITE_ERROR:
+			log_request(session, session->return_code);
+			break;
+		case ec_FORCE_QUIT:
 			log_system(session, "Client kicked");
 			break;
-		case rr_SQL_INJECTION:
+		case ec_SQL_INJECTION:
 			if ((session->config->ban_on_sqli > 0) && (ip_allowed(&(session->ip_address), session->config->banlist_mask) != deny)) {
 				ban_ip(&(session->ip_address), session->config->ban_on_sqli, session->config->kick_on_ban);
 				log_system(session, "Client banned because of SQL injection");
@@ -1250,6 +1265,12 @@ void handle_request_result(t_session *session, int result) {
 			send_code(session);
 			log_request(session, 400);
 			break;
+		default:
+			if (session->data_sent == false) {
+				session->return_code = 500;
+				send_code(session);
+			}
+	} else switch (result) {
 		case 200:
 			break;
 		case 201:
@@ -1399,7 +1420,6 @@ void connection_handler(t_session *session) {
 		close_socket(session);
 	} else {
 		close(session->client_socket);
-		session->socket_open = false;
 	}
 
 #ifdef HAVE_MONITOR
@@ -1834,13 +1854,13 @@ int run_server(t_settings *settings) {
 		return -1;
 	}
 
-	if (read_mimetypes(config->mimetype_config, &(config->mimetype)) == -1) {
+	if (read_mimetypes(config->mimetype_config, &(config->mimetype), settings->config_check) == -1) {
 		fprintf(stderr, "Error while reading mimetype configuration.\n");
 		return -1;
 	}
 
 	if (settings->config_check) {
-		printf("Configuration oke.\n");
+		printf("Configuration OK.\n");
 		return 0;
 	}
 
@@ -1929,12 +1949,12 @@ int run_server(t_settings *settings) {
 
 	/* Create work directory
 	 */
-	if (mkdir(config->work_directory, 0700) == -1) {
+	if (mkdir(config->work_directory, S_IRWXU) == -1) {
 		if (errno != EEXIST) {
 			fprintf(stderr, "Error creating work directory '%s'\n", config->work_directory);
 			return -1;
 #ifndef CYGWIN
-		} else if (chmod(config->work_directory, 0700) == -1) {
+		} else if (chmod(config->work_directory, S_IRWXU) == -1) {
 			fprintf(stderr, "Can't change access permissions of work directory '%s'\n", config->work_directory);
 			return -1;
 #endif
@@ -1951,7 +1971,7 @@ int run_server(t_settings *settings) {
 
 	/* Create the upload directory for PUT requests
 	 */
-	if (mkdir(config->upload_directory, 0733) == -1) {
+	if (mkdir(config->upload_directory, S_IRWXU | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH) == -1) {
 		if (errno != EEXIST) {
 			fprintf(stderr, "Error while creating UploadDirectory '%s'\n", config->upload_directory);
 			return -1;
@@ -1975,7 +1995,7 @@ int run_server(t_settings *settings) {
 		}
 	}
 
-	if ((status.st_mode & 07777) != access_rights) {
+	if ((status.st_mode & (S_IRWXU | S_IRWXG | S_IRWXO)) != access_rights) {
 		if (chmod(config->upload_directory, access_rights) == -1) {
 			fprintf(stderr, "Can't change access permissions of UploadDirectory '%s'.\n", config->upload_directory);
 			return -1;
@@ -1986,12 +2006,12 @@ int run_server(t_settings *settings) {
 #ifdef HAVE_MONITOR
 	/* Create monitor cache directory
 	 */
-	if (mkdir(config->monitor_directory, 0700) == -1) {
+	if (mkdir(config->monitor_directory, S_IRWXU) == -1) {
 		if (errno != EEXIST) {
 			fprintf(stderr, "Error creating monitor directory '%s'\n", config->monitor_directory);
 			return -1;
 #ifndef CYGWIN
-		} else if (chmod(config->monitor_directory, 0700) == -1) {
+		} else if (chmod(config->monitor_directory, S_IRWXU) == -1) {
 			fprintf(stderr, "Can't change access permissions of monitor directory '%s'\n", config->monitor_directory);
 			return -1;
 #endif
