@@ -24,7 +24,7 @@
 
 /* Convert a string to an integer.
  */
-int str2int(const char *str) {
+int str2int(char *str) {
 	int i = 0, value = 0;
 
 	if (str == NULL) {
