@@ -71,7 +71,7 @@ static t_host *new_host(void) {
 	host->use_gz_file        = false;
 	host->access_list        = NULL;
 	host->alter_list         = NULL;
-	host->alter_fmode        = 0640;
+	host->alter_fmode        = S_IRUSR | S_IWUSR | S_IRGRP;
 	host->run_on_alter       = NULL;
 	host->login_message      = "Private page";
 	host->passwordfile       = NULL;
@@ -995,10 +995,12 @@ static bool system_setting(char *key, char *value, t_config *config) {
 			return true;
 		}
 	} else if (strcmp(key, "workdirectory") == 0) {
-		if (valid_directory(value)) {
-			if ((config->work_directory = strdup(value)) != NULL) {
-				return true;
-			}
+		if (valid_directory(value) == false) {
+			return false;
+		} else if (strchr(value + 1, '/') == NULL) {
+			return false;
+		} else if ((config->work_directory = strdup(value)) != NULL) {
+			return true;
 		}
 	} else if (strcmp(key, "wrapusercgi") == 0) {
 		if (parse_yesno(value, &(config->wrap_user_cgi)) == 0) {
@@ -1226,7 +1228,7 @@ static bool host_setting(char *key, char *value, t_host *host) {
 		if ((host->no_extension_as = strdup(value)) != NULL) {
 			return true;
 		}
-	} else if (strcmp(key, "preventcsrf") == 0) {
+	} else if ((strcmp(key, "preventcsrf") == 0) || (strcmp(key, "preventxsrf") == 0)) {
 		if (parse_yesno(value, &(host->prevent_csrf)) == 0) {
 			return true;
 		}
@@ -1242,17 +1244,6 @@ static bool host_setting(char *key, char *value, t_host *host) {
 		if (parse_charlist(value, &(host->required_binding)) == 0) {
 			return true;
 		}
-	} else if (strcmp(key, "usefastcgi") == 0) {
-		if (parse_charlist(value, &(host->fast_cgi)) != -1) {
-			host->execute_cgi = true;
-			return true;
-		}
-#ifdef HAVE_TOOLKIT
-	} else if (strcmp(key, "usetoolkit") == 0) {
-		if (parse_charlist(value, &(host->toolkit_rules)) == 0) {
-			return true;
-		}
-#endif
 	} else if (strcmp(key, "secureurl") == 0) {
 		if (parse_yesno(value, &(host->secure_url)) == 0) {
 			return true;
@@ -1265,6 +1256,17 @@ static bool host_setting(char *key, char *value, t_host *host) {
 		if (parse_yesno(value, &(host->trigger_on_cgi_status)) == 0) {
 			return true;
 		}
+	} else if (strcmp(key, "usefastcgi") == 0) {
+		if (parse_charlist(value, &(host->fast_cgi)) != -1) {
+			host->execute_cgi = true;
+			return true;
+		}
+#ifdef HAVE_TOOLKIT
+	} else if (strcmp(key, "usetoolkit") == 0) {
+		if (parse_charlist(value, &(host->toolkit_rules)) == 0) {
+			return true;
+		}
+#endif
 	} else if (strcmp(key, "userwebsites") == 0) {
 		if (parse_yesno(value, &(host->user_websites)) == 0) {
 			return true;
