@@ -103,6 +103,7 @@ void init_session(t_session *session) {
 	}
 	session->request_limit = true;
 	session->force_quit = false;
+	session->keep_alive = false;
 	session->kept_alive = 0;
 #ifdef HAVE_SSL
 	session->ssl_data = NULL;
@@ -124,6 +125,8 @@ void init_session(t_session *session) {
 void reset_session(t_session *session) {
 	int size;
 
+	session->error_cause = ec_NONE;
+
 	sfree(session->file_on_disk);
 #ifdef CIFS
 	sfree(session->extension);
@@ -523,7 +526,7 @@ int prevent_xss(t_session *session) {
 /* Prevent SQL injection
  */
 int prevent_sqli(t_session *session, char *str, int length, char **newstr) {
-	int changed = 0, i, add, left, newlength = 0;
+	int changed = 0, i, add, left, newlength = 0, extra_buffer = 128;
 	char *temp, value;
 	short low, high;
 
@@ -531,7 +534,7 @@ int prevent_sqli(t_session *session, char *str, int length, char **newstr) {
 		return 0;
 	}
 
-	add = left = 32;
+	add = left = extra_buffer;
 	if ((*newstr = (char*)malloc(length + add + 1)) == NULL) {
 		return -1;
 	}
@@ -541,19 +544,19 @@ int prevent_sqli(t_session *session, char *str, int length, char **newstr) {
 		if ((value == '%') && (i + 2 < length)) {
 			if ((high = hex_to_int(*(str + i + 1))) != -1) {
 				if ((low = hex_to_int(*(str + i + 2))) != -1) {
-					value = (char)(high<<4) + low;
+					value = (char)(high << 4) + low;
 				}
 			}
 		}
 
 		if ((value == '\'') || (value == '`') || (value == '\\')) {
-			if (changed == 0) {
-				log_exploit_attempt(session, "XSS", session->vars);
+			if (changed++ == 0) {
+				log_exploit_attempt(session, "SQLi", session->vars);
 			}
 
-			*(*newstr + newlength++) = '\\';
-			if (left-- == 0) {
-				add += left = 32;
+			*(*newstr + (newlength++)) = '\\';
+			if (--left == 0) {
+				add += left = extra_buffer;
 				if ((temp = (char*)realloc(*newstr, length + add + 1)) == NULL) {
 					free(*newstr);
 					return -1;
@@ -561,7 +564,6 @@ int prevent_sqli(t_session *session, char *str, int length, char **newstr) {
 					*newstr = temp;
 				}
 			}
-			changed++;
 		}
 		*(*newstr + (newlength++)) = *(str + i);
 	}
