@@ -23,6 +23,15 @@
 #define MAX_CHILD_ID        30000
 #define OUTPUT_BUFFER_SIZE      2 * KILOBYTE
 
+#define ec_NONE                  0
+#define ec_SOCKET_READ_ERROR    -1
+#define ec_SOCKET_WRITE_ERROR   -2
+#define ec_MAX_REQUESTSIZE      -3
+#define ec_TIMEOUT              -4
+#define ec_CLIENT_DISCONNECTED  -5
+#define ec_FORCE_QUIT           -6
+#define ec_SQL_INJECTION        -7
+
 typedef enum { no_cgi, binary, script, fastcgi} t_cgi_type;
 typedef enum { unknown, GET, POST, HEAD, OPTIONS, TRACE, PUT, DELETE, unsupported } t_req_method;
 typedef enum { missing_slash, require_ssl, location } t_cause_of_301;
@@ -30,6 +39,7 @@ typedef enum { missing_slash, require_ssl, location } t_cause_of_301;
 typedef struct type_session {
 	t_config        *config;
 
+	int             error_cause;
 	time_t          time;
 	int             client_id;
 	int             client_socket;
