@@ -242,7 +242,7 @@ int send_file(t_session *session) {
 
 	/* Range
 	 */
-	if ((*(session->http_version + 7) == '1') && (session->handling_error == false)) {
+	if (session->handling_error == false) {
 		if ((range = get_headerfield("Range:", session->headerfields)) != NULL) {
 			if (strncmp(range, "bytes=", 6) == 0) {
 				if ((range = strdup(range + 6)) == NULL) {
@@ -267,7 +267,7 @@ int send_file(t_session *session) {
 					} else {
 						if ((send_begin = str2int(range_end)) >= 0) {
 							/* bytes=-XX */
-							send_begin = file_size - send_begin;
+							send_begin = file_size - send_begin - 1;
 							session->return_code = 206;
 						}
 					}
@@ -500,7 +500,7 @@ int execute_cgi(t_session *session) {
 
 #ifdef CYGWIN
 	if ((session->config->platform == windows) && (session->cgi_type == binary)) {
-		chmod(session->file_on_disk, 0755);
+		chmod(session->file_on_disk, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
 	}
 #endif
 
@@ -562,7 +562,8 @@ int execute_cgi(t_session *session) {
 				default:
 					if (session->config->ban_on_sqli > 0) {
 						free(line);
-						return rr_SQL_INJECTION;
+						session->error_cause = ec_SQL_INJECTION;
+						return -1;
 					}
 					if (register_tempdata(&(session->tempdata), line, tc_data) == -1) {
 						free(line);
@@ -581,7 +582,8 @@ int execute_cgi(t_session *session) {
 				default:
 					if (session->config->ban_on_sqli > 0) {
 						free(line);
-						return rr_SQL_INJECTION;
+						session->error_cause = ec_SQL_INJECTION;
+						return -1;
 					}
 					if (register_tempdata(&(session->tempdata), line, tc_data) == -1) {
 						free(line);
@@ -601,7 +603,8 @@ int execute_cgi(t_session *session) {
 				default:
 					if (session->config->ban_on_sqli > 0) {
 						free(line);
-						return rr_SQL_INJECTION;
+						session->error_cause = ec_SQL_INJECTION;
+						return -1;
 					}
 					if (register_tempdata(&(session->tempdata), line, tc_data) == -1) {
 						free(line);
@@ -1001,7 +1004,7 @@ int handle_trace_request(t_session *session) {
 
 	header = session->headerfields;
 	while (header != NULL) {
-		body_size += strlen(header->data) + 1;
+		body_size += header->length + 1;
 		header = header->next;
 	}
 
@@ -1044,17 +1047,16 @@ int handle_trace_request(t_session *session) {
 
 		header = session->headerfields;
 		while (header != NULL) {
-			len = strlen(header->data);
-			*(header->data + len) = '\n';
-			if (send_buffer(session, header->data, len + 1) == -1) {
-				*(header->data + len) = '\0';
-				result = -2;
+			if (send_buffer(session, header->data, header->length) == -1) {
+				result = -1;
+				break;
+			} else if (send_buffer(session, "\n", 1) == -1) {
+				result = -1;
 				break;
 			}
-			*(header->data + len) = '\0';
 			header = header->next;
 		}
-		if (result == -2) {
+		if (result == -1) {
 			break;
 		}
 	
