@@ -300,9 +300,8 @@ bool can_transform_with_xslt(t_session *session) {
 		fclose(fp);
 		session->xslt_file = xslt;
 		return true;
-	} else {
-		free(xslt);
 	}
+	free(xslt);
 
 	/* Check for XSLT existence: index.xslt in directory
 	 */
@@ -319,9 +318,8 @@ bool can_transform_with_xslt(t_session *session) {
 		fclose(fp);
 		session->xslt_file = xslt;
 		return true;
-	} else {
-		free(xslt);
 	}
+	free(xslt);
 
 	/* Check for XSLT existence: /index.xslt
 	 */
@@ -334,9 +332,8 @@ bool can_transform_with_xslt(t_session *session) {
 		fclose(fp);
 		session->xslt_file = xslt;
 		return true;
-	} else {
-		free(xslt);
 	}
+	free(xslt);
 
 	return false;
 }
@@ -360,6 +357,11 @@ static int apply_xslt_sheet(t_session *session, xmlDocPtr data_xml) {
 
 	/* Read XSLT sheet
 	 */
+	if (session->xslt_file == NULL) {
+		log_file_error(session, session->xslt_file, "XSLT file not set");
+		return 500;
+	}
+
 	if ((style_xml = xmlReadFile(session->xslt_file, NULL, 0)) == NULL) {
 		log_file_error(session, session->xslt_file, "XSLT file contains invalid XML");
 		return 500;
@@ -697,9 +699,15 @@ int show_index(t_session *session) {
 			sfree(link);
 			remove_filelist(filelist);
 			return -1;
-		} else {
-			sfree(link);
+		} else if (file->is_dir) {
+			if (add_str(&text_xml, &text_max, XML_CHUNK_LEN, &text_size, "/") == -1) {
+				free(text_xml);
+				sfree(link);
+				remove_filelist(filelist);
+				return -1;
+			}
 		}
+		sfree(link);
 
 		if (xml_special_chars(file->name, &link) == -1) {
 			free(text_xml);
